# encoding: utf-8

import cgi
import os

from pyroutes.http.cookies import RequestCookieHandler

try:
    import cStringIO as StringIO
except ImportError:
    import StringIO

class Request(object):

    def __init__(self, environment):
        self.ERRORS = []
        self.ENV = environment

        # Initialize GET
        self.GET = self.extract_get_data(environment)

        # Initialize POST and FILES
        self.POST = {}
        self.FILES = {}
        self.extract_post_data(environment)

        self.COOKIES = RequestCookieHandler(environment)


    def __repr__(self):
        return "GET: %s\nPOST: %s\nCOOKIES: %s\nFILES: %s" % \
            (self.GET, self.POST, self.COOKIES._raw_cookies, self.FILES.keys())

    def extract_post_data(self, environment):
        data = {}

        # Copy enviroment so we dont get GET-variables in the result.
        env = environment.copy()
        env['QUERY_STRING'] = ''

        if env.get('REQUEST_METHOD', 'GET') == 'POST':
            length = int(env.get('CONTENT_LENGTH', 0))
            if length > 5*1024:
                input_buffer = os.tmpfile()
            else:
                input_buffer = StringIO.StringIO()
            read_bytes = 1024*32
            while length:
                if read_bytes > length:
                    read_bytes = length
                input_buffer.write(environment['wsgi.input'].read(read_bytes))
                length = length - read_bytes
                if length < 0:
                    length = 0

            if hasattr(input_buffer, 'reset'):
                input_buffer.reset()
            else:
                input_buffer.seek(0)

            _data = cgi.FieldStorage(
                fp=input_buffer,
                environ=env,
                keep_blank_values=False
            )
            for key in _data.keys():
                value = self._parse_field(_data[key], key, _data)
                if value is not None:
                    self._assign_field_to_section(key, value)

            input_buffer.close()
        return data

    def extract_get_data(self, environment):
        ret_dict = {}
        for (key, value) in cgi.parse_qsl(environment.get('QUERY_STRING', '')):
            if key in ret_dict:
                if not isinstance(ret_dict[key], list):
                    ret_dict[key] = [ret_dict[key]]
                ret_dict[key].append(value)
            else:
                ret_dict[key] = value
        return ret_dict

    def _assign_field_to_section(self, key, value):
        if isinstance(value, list):
            for val in value:
                self._assign_field_to_section(key, val)
        else:
            if isinstance(value, tuple) and value[1] and \
              (isinstance(value[1], file) or  hasattr(value[1], 'read')):

                # If an existing value exists for this key, convert to list-result
                if key in self.FILES and not isinstance(self.FILES[key], list):
                    self.FILES[key] = [self.FILES[key]]

                if key in self.FILES and isinstance(self.FILES[key], list):
                    self.FILES[key].append(value)
                else:
                    self.FILES[key] = value
            elif isinstance(value, basestring):
                # If an existing value exists for this key, convert to list-result
                if key in self.POST and not isinstance(self.POST[key], list):
                    self.POST[key] = [self.POST[key]]

                if key in self.POST and isinstance(self.POST[key], list):
                    self.POST[key].append(value)
                else:
                    self.POST[key] = value

    def _parse_field(self, field, key, data):
        value = None
        if isinstance(field, list):
            value = [self._parse_field(f, key, data) for f in field]
        if hasattr(field, 'filename') and field.filename:
            if field.file:
                value = (field.filename, field.file)
            else:
                value = (field.filename, StringIO.StringIO(data.getvalue(key)))
        else:
            if isinstance(data.getvalue(key), basestring):
                try:
                    value = unicode(data.getvalue(key), 'utf-8')
                except UnicodeDecodeError:
                    # If we can't understand the data as utf, try latin1
                    value = unicode(data.getvalue(key), 'iso-8859-1')

        return value



#!/usr/bin/env python
# encoding: utf-8
"""
"""
import os
import xmltemplate
from pyroutes import settings

class TemplateRenderer(object):
    """
    A small wrapper for doing basic template includes with xml-template

    The constructor takes two arguments, a base template and an inclusion parameter.
    The base template must have a <t:contents /> node in it which will be replaced with
    the contents of the included template. The contents identifier can
    be overriden with the second constructor argument, `inclusion_param`.
    If None is given as the base template, the `render`-method will not try
    to do any inclusion and only render the given template.
    """

    def __init__(self, base_template=None, inclusion_param='contents', template_dir=None):
        """
        """
        self.base_template = base_template
        self.inclusion_param = inclusion_param
        if template_dir != None:
            self.template_dir = template_dir
        elif settings.TEMPLATE_DIR:
            self.template_dir = settings.TEMPLATE_DIR
        else:
            self.template_dir = ''

    def render(self, template, data):
        """
        """
        if self.base_template:
            doc = xmltemplate.process_file(os.path.join(self.template_dir, template), data, False)
            data[self.inclusion_param] = doc
            master = xmltemplate.process_file(os.path.join(self.template_dir, self.base_template), data);
        else:
            master = xmltemplate.process_file(os.path.join(self.template_dir, template), data)
        return master.toxml().encode("utf-8")



#! /usr/bin/python
import xml.dom.minidom

def process_file(filename, obj, clean = True):
    doc = xml.dom.minidom.parse(filename)
    process(doc, obj, clean)
    return doc

def process(node, obj, clean = True):
    if isinstance(obj, basestring):            # overwrite
        while not node.firstChild is None:
            node.removeChild(node.firstChild)
        doc = _get_document_element(node)
        node.appendChild(doc.createTextNode(obj))
    elif isinstance(obj, xml.dom.Node):    # overwrite
        while not node.firstChild is None:
            node.removeChild(node.firstChild)

        if isinstance(obj, xml.dom.minidom.Document):
            obj = obj.documentElement

        newobj = obj.cloneNode(True)
        node.appendChild(newobj)

        process(newobj, {}, clean)
    elif isinstance(obj, dict):         # substitute
        for child in node.childNodes:
            processed = False

            if child.nodeType == xml.dom.Node.ELEMENT_NODE:
                tag_id = None

                attrs = child.attributes
                attrs_to_remove = []
                if not attrs is None:
                    for i in range(attrs.length):
                        attr = attrs.item(i)
                        if attr.namespaceURI == "http://template.sesse.net/" \
                            and attr.localName == "id":
                            tag_id = attr.value
                            if clean:
                                attrs_to_remove.append(attr.name)
                        if attr.name.startswith("xmlns:") and \
                           attr.value == "http://template.sesse.net/" and clean:
                            attrs_to_remove.append(attr.name)

                    for attr in attrs_to_remove:
                        if child.hasAttribute(attr):
                            child.removeAttribute(attr)


                # check all substitutions to see if we found anything
                # appropriate
                for key in obj.keys():
                    if key.startswith(child.tagName + "/"):
                        child.setAttribute(key.split("/")[1], obj[key])
                    elif (not tag_id is None) and \
                      key.startswith("#" + tag_id + "/"):
                        child.setAttribute(key.split("/")[1], obj[key])

                    if not processed:
                        if key == child.localName or ((not tag_id is None) and \
                           key == "#" + tag_id):
                            process(child, obj[key], clean)
                            processed = True

            if not processed:
                process(child, obj, clean)
    elif isinstance(obj, list):         # repeat
        doc = _get_document_element(node)
        frag = doc.createElement("temporary-fragment")    # ugh

        while not node.firstChild is None:
            child = node.firstChild
            node.removeChild(child)
            frag.appendChild(child)

        for instance in obj:
            if instance is not None:
                newnode = frag.cloneNode(True)
                node.appendChild(newnode)
                process(newnode, instance, clean)
                if clean:
                    _clean(newnode)

        # remove all the <fragment> tags

        children_to_remove = []
        for child in node.childNodes:
            if isinstance(child, xml.dom.minidom.Element) and \
               child.tagName == 'temporary-fragment':
                while not child.firstChild is None:
                    child2 = child.firstChild
                    child.removeChild(child2)
                    node.appendChild(child2)
                children_to_remove.append(child)

        for child in children_to_remove:
            node.removeChild(child)

    if clean:
        _clean(node)

def alternate(tag, array, *elems):
    i = 0
    for ref in array:
        if ref is not None:
            ref[tag] = elems[i % len(elems)]
            i = i + 1
        
    return array

def _clean(node):
    if node.nodeType == xml.dom.Node.ELEMENT_NODE and \
       node.namespaceURI == "http://template.sesse.net/":
        # as this is a dummy node, we want to remove it and move everything
        # further up after we've done any required replacements
        parent = node.parentNode

        while not node.firstChild is None:
            child = node.firstChild
            node.removeChild(child)
            parent.insertBefore(child, node)

        parent.removeChild(node)

# ugh
def _get_document_element(node):
    if node.parentNode is None:
        return node
    else:
        return _get_document_element(node.parentNode)



import os

"""Used for importing settings, and defining default settings"""

# Default template dir, relative to this imported __file__
BUILTIN_TEMPLATES_DIR = os.path.join(
    os.path.dirname(__file__),
    'default_templates'
)
BUILTIN_BASE_TEMPLATE = 'base.xml'

# This should have an override for development servers
DEBUG = False

# For setting the default content type
DEFAULT_CONTENT_TYPE = 'text/html; charset=utf-8'

# Secret key for crypto. CHANGE THIS IS pyroutes_settings!
SECRET_KEY = 'asdfnaj2308sydfahli37flas36al9gaiufw'

# Location for templates. Used by TemplateRenderer
TEMPLATE_DIR = None # /foo/bar/templates/

# Attempt to get custom settings. Not obligatory.
try:
    from pyroutes_settings import *
except ImportError:
    pass



#!/usr/bin/env python
# encoding: utf-8
"""
http.py

A collection of Response classes for pyroutes
"""

import os

from pyroutes.template import TemplateRenderer
from pyroutes.http.cookies import ResponseCookieHandler
from pyroutes import settings

try:
    from httplib import responses
except ImportError:
    responses = {
        100: 'Continue',
        101: 'Switching Protocols',
        200: 'OK',
        201: 'Created',
        202: 'Accepted',
        203: 'Non-Authoritative Information',
        204: 'No Content',
        205: 'Reset Content',
        206: 'Partial Content',
        300: 'Multiple Choices',
        301: 'Moved Permanently',
        302: 'Found',
        303: 'See Other',
        304: 'Not Modified',
        305: 'Use Proxy',
        306: '(Unused)',
        307: 'Temporary Redirect',
        400: 'Bad Request',
        401: 'Unauthorized',
        402: 'Payment Required',
        403: 'Forbidden',
        404: 'Not Found',
        405: 'Method Not Allowed',
        406: 'Not Acceptable',
        407: 'Proxy Authentication Required',
        408: 'Request Timeout',
        409: 'Conflict',
        410: 'Gone',
        411: 'Length Required',
        412: 'Precondition Failed',
        413: 'Request Entity Too Large',
        414: 'Request-URI Too Long',
        415: 'Unsupported Media Type',
        416: 'Requested Range Not Satisfiable',
        417: 'Expectation Failed',
        500: 'Internal Server Error',
        501: 'Not Implemented',
        502: 'Bad Gateway',
        503: 'Service Unavailable',
        504: 'Gateway Timeout',
        505: 'HTTP Version Not Supported'}

class Response(object):
    """
    A wrapper class for a response to a route. Takes
    a content, headers and status_code parameter.
    headers should be passed in as a list of tuples.
    If default_content_header is set and no Content-Type header is set,
    settings.DEFAULT_CONTENT_TYPE is added as Content-Type.
    """
    def __init__(self, content=None, headers=None, status_code='200 OK',
            default_content_header=True):
        if content is None:
            self.content = []
        else:
            self.content = content

        header_names = []
        if headers:
            header_names = map(lambda h: h[0], headers);

        self.headers = []
        if default_content_header and 'Content-Type' not in header_names:
            # Whether or not this is a safe operation may be python/wsgiref
            # version dependent. Should be tested and documented.
            self.headers.append(('Content-Type', settings.DEFAULT_CONTENT_TYPE))
        if not headers is None:
            self.headers += headers

        if status_code in responses:
            self.status_code = "%s %s" % (status_code, responses[status_code])
        else:
            self.status_code = status_code

        self.cookies = ResponseCookieHandler()

class Redirect(Response):
    """
    A redirect shortcut class for redirection responses
    """

    def __init__(self, location):
        self.content = "redirect"
        self.headers = [('Location', location)]
        self.status_code = "302 See Other"
        self.cookies = ResponseCookieHandler()

class HttpException(Exception):
    def __init__(self):
        if not hasattr(self, 'code'):
            raise TypeError('You tried to instanciate HttpException. ' \
                    + 'Please, only create instances of Http{403,404,500}.')

        self.template_variable = 'TEMPLATE_%d' % self.code
        self.template_filename = '%d.xml' % self.code
        self.status_code = {
            403: '403 Forbidden',
            404: '404 Not Found',
            500: '500 Server Error'
        }[self.code]

        if hasattr(settings, self.template_variable):
            if hasattr(settings, 'CUSTOM_BASE_TEMPLATE'):
                self.templaterenderer = TemplateRenderer(
                    settings.CUSTOM_BASE_TEMPLATE
                )
            else:
                self.templaterenderer = TemplateRenderer()
            self.template = getattr(settings, self.template_variable)
        else:
            self.templaterenderer = TemplateRenderer(
                settings.BUILTIN_BASE_TEMPLATE,
                template_dir=settings.BUILTIN_TEMPLATES_DIR
            )
            self.template = self.template_filename

    def get_response(self, path, **kwargs):
        template_data = {
            'request': path,
            'title': self.status_code
        }
        template_data.update(kwargs)
        document = self.templaterenderer.render(self.template, template_data)
        return Response(document, status_code=self.status_code)

class Http403(HttpException):
    code = 403

class Http404(HttpException):
    code = 404

class Http500(HttpException):
    code = 500



import datetime
import mimetypes
import os
import posixpath
import time

os.path = posixpath

from pyroutes import settings
from pyroutes.template import TemplateRenderer
from pyroutes.http.response import Response, Redirect, Http403, Http404
from pyroutes.contrib import autoreload

from wsgiref.simple_server import make_server
from wsgiref.util import FileWrapper

def devserver(application, port=8001, address='0.0.0.0', auto_reload=True):
    """
    Simple development server for rapid development. Use to create a simple web
    server. For testing purposes only. Has no builting handling of file
    serving, use fileserver in this class. Typical usage::

        from pyroutes import route, application, utils
        #<define routes>
        if __name__ == '__main__':
            utils.devserver(application)

    This starts a server listening on all interfaces, port 8001. It
    automatically reloads modified files.
    """

    def server_thread():
        httpd = make_server(address, port, application)
        print "Starting server on %s port %d..." % (address, port)
        httpd.serve_forever()
    if auto_reload:
        autoreload.main(server_thread)
    else:
        server_thread()

def fileserver(request):
    """
    Simple file server for development servers. Not for use in production
    environments. Typical usage::

        from pyroutes import route, utils
        route('/media')(utils.fileserver)

    That will add the fileserver to the route /media. If DEV_MEDIA_BASE is
    defined in settings, host files from this folder. Otherwise, use current
    working directory.

    .. note::
        DEV_MEDIA_BASE and route path is concatenated, i.e. if you use
        '/srv/media' for as the media base, and map the route to '/files', all
        files will be looked for in '/srv/media/files'
    """

    path = request.ENV['PATH_INFO']
    path_list = path.lstrip('/').split('/')

    # Do not expose entire file system
    if '..' in path_list:
        raise Http404

    if hasattr(settings, 'DEV_MEDIA_BASE'):
        path = os.path.join(settings.DEV_MEDIA_BASE, *path_list)
    else:
        path = os.path.join('.', *path_list)

    if not os.path.exists(path):
        raise Http404

    if not os.access(path, os.R_OK):
        raise Http403

    modified = datetime.datetime.fromtimestamp(os.path.getmtime(path))
    if 'HTTP_IF_MODIFIED_SINCE' in request.ENV:
        modified = datetime.datetime.strftime(modified, "%a, %d %b %Y %H:%M:%S")
        if request.ENV.get('HTTP_IF_MODIFIED_SINCE') == modified:
            return Response(status_code='304 Not Modified',
                    default_content_header=False)
    modified = datetime.datetime.strftime(modified, "%a, %d %b %Y %H:%M:%S")

    headers = [
        ('Last-Modified', modified),
    ]

    if os.path.isdir(path):
        if not path.endswith('/'):
            return Redirect(path + '/')

        listing = []
        files = []
        for file in sorted(os.listdir(path)):
            if os.path.isdir(os.path.join(path, file)):
                listing.append({'li': {'a': file + "/", 'a/href': file + "/"}})
            else:
                files.append({'li': {'a': file, 'a/href': file}})
        # Done to list folders before files
        listing += files

        template_data = {
            'file_list': listing,
            'title': 'Listing of %s' % path
        }

        templaterenderer = TemplateRenderer(
            settings.BUILTIN_BASE_TEMPLATE,
            template_dir=settings.BUILTIN_TEMPLATES_DIR
        )
        return Response(
            templaterenderer.render(
                os.path.join('fileserver', 'directory_listing.xml'),
                template_data
            ),
            headers
        )

    contenttype = mimetypes.guess_type(path)[0] or "application/octet-stream"
    file = FileWrapper(open(path))
    size = os.path.getsize(path)

    headers.append(('Content-Type', contenttype))
    headers.append(('Content-Length', str(size)))

    return Response(file, headers=headers)



#encoding: utf-8

import hmac
try:
    from hashlib import sha1
except ImportError:
    import sha as sha1

from pyroutes import settings

class RequestCookieHandler(object):
    """
    Class for handling all things regarding to cookies in pyroutes request objects.
    """

    def __init__(self, environ=None):
        self._raw_cookies = {}

        if environ and 'HTTP_COOKIE' in environ:
            for (key, value) in [(c[:c.find('=')].strip(), c[c.find('=')+1:].strip()) for c in environ.get('HTTP_COOKIE', '').split(';')]:
                self._raw_cookies[key] = value

    def get_cookie(self, key):
        """
        Return a signed cookie value if it exists.
        """
        if key in self._raw_cookies:
            if '%s_hash' % key in self._raw_cookies:
                cookie_hash = self._raw_cookies['%s_hash' % key]
                value_hash = hmac.HMAC(settings.SECRET_KEY, key + self._raw_cookies[key], sha1).hexdigest()
                if cookie_hash == value_hash:
                    return self._raw_cookies[key]
                else:
                    raise CookieHashInvalid(key, 'Cookie modified')
            else:
                raise CookieHashMissing(key, 'Cookie hash missing')
        else:
            return None

    def get_unsigned_cookie(self, key):
        """
        Return an unsigned cookie value. (Plain old-fashion cookie reading)
        """
        if key in self._raw_cookies:
            return self._raw_cookies[key]
        else:
            return None


class ResponseCookieHandler(object):
    """
    Handle cookie adding to request.
    """

    def __init__(self):
        self.cookie_headers = []

    def add_cookie(self, key, value, expires=None):
        cookie_hash = hmac.HMAC(settings.SECRET_KEY, key + value, sha1).hexdigest()
        if expires:
            exp = expires.strftime("%a, %d-%b-%Y %H:%M:%S GMT")
            self.cookie_headers.append(('Set-Cookie', '%s=%s; expires=%s' % (key, value, exp)))
            self.cookie_headers.append(('Set-Cookie', '%s_hash=%s; expires=%s' % (key, cookie_hash, exp)))
        else:
            self.cookie_headers.append(('Set-Cookie', '%s=%s' % (key, value)))
            self.cookie_headers.append(('Set-Cookie', '%s_hash=%s' % (key, cookie_hash)))

    def add_unsigned_cookie(self, key, value, expires=None):
        if expires:
            exp = expires.strftime("%a, %d-%b-%Y %H:%M:%S GMT")
            self.cookie_headers.append(('Set-Cookie', "%s=%s; expires=%s" % (key, value, exp)))
        else:
            self.cookie_headers.append(('Set-Cookie', "%s=%s" % (key, value)))

    def del_cookie(self, key):
        self.cookie_headers.append(('Set-Cookie', "%s=null; expires=Thu, 01-Jan-1970 00:00:01 GMT" % key))
        self.cookie_headers.append(('Set-Cookie', "%s_hash=null; expires=Thu, 01-Jan-1970 00:00:01 GMT" % key))

class CookieHashMissing(LookupError):
    pass

class CookieHashInvalid(ValueError):
    pass



#!/usr/bin/env python
# encoding: utf-8

from distutils.core import setup
import os

setup(
    name = "pyroutes",
    version = "0.2.2",
    url = 'http://github.com/klette/pyroutes',
    license = 'GPLv2',
    description = "A small WSGI wrapper for creating small python web apps",
    long_description = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    author = 'Kristian Klette',
    author_email = 'klette@samfundet.no',
    packages = ['pyroutes','pyroutes.http', 'pyroutes.template', 'pyroutes.contrib'],
    requires = ['wsgiref'],
    package_data = {'pyroutes': ['default_templates/*.xml', 'default_templates/fileserver/*.xml']},

    classifiers = [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Operating System :: OS Independent',
        ]

)




#!/usr/bin/env python
#encoding: utf-8

"""
Main pyroutes module

This module handles all the dispatching services for pyroutes.
"""

from pyroutes.http.response import HttpException, Http404, Http500
from pyroutes.http.request import Request
from pyroutes import settings

from wsgiref.util import shift_path_info
import cgi
import os
import sys
import traceback

__request__handlers__ = {}

def route(path):
    """
    Routes define which methods handles requests to certain paths, and are defined
    using the `@route`-decorator. The decorator takes one argument that defines which
    path the method is used for. The decorated function recieves one argument from pyroutes
    containing a `Request`-instance with all the information for that particular request.

    **Defining routes**

    This is a simple example route ::

      from pyroutes import route
      from pyroutes.http.response import Response

      @route('/')
      def index(request):
          return Response('Hello world')


    **How routes are matched to paths.**

    One property of the routes are that matches are done on an best effort basis, starting
    from the top of the tree and going down. This results in handler being delt request for
    their defined path and every path over it. This is true for all paths except the
    root-handler ('/'). Let's have a look at some examples. ::

        In [7]: @pyroutes.route('/')
            ...: def foo(req):
            ...:     return None
            ...:

        In [8]: pyroutes.find_request_handler('/')
        Out[8]: <function foo at 0x2688de8>

        In [9]: pyroutes.find_request_handler('/foobar')

        In [10]: @pyroutes.route('/foo')
            ...: def bar(req):
            ...:     return None
            ...:

        In [11]: pyroutes.find_request_handler('/foo/')
        Out[11]: <function bar at 0x27881b8>

        In [12]: pyroutes.find_request_handler('/foo/bar')
        Out[12]: <function bar at 0x27881b8>

        In [13]: pyroutes.find_request_handler('/foo/bar/baz')
        Out[13]: <function bar at 0x27881b8>
    """

    def decorator(func):
        """
        See the pyroutes.route-docstring
        """
        if path in __request__handlers__:
            raise ValueError("Tried to redefine handler for %s with %s" % \
                    (path, func))
        __request__handlers__[path] = func
        return func
    return decorator

def create_request_path(environ):
    """
    Returns a tuple consisting of the individual request parts
    """
    path = shift_path_info(environ)
    request = []
    if not path:
        request = ['/']
    else:
        while path:
            request.append(path)
            path = shift_path_info(environ)
    return request

def find_request_handler(current_path):
    """
    Locates the handler for the specified path. Return None if not found.
    """
    handler = None
    while handler is None:
        if current_path in __request__handlers__:
            handler = __request__handlers__[current_path]
            break
        current_path = current_path[:current_path.rfind("/")]
        if not current_path:
            return None
    return handler

def reverse_url(handler_name):
    """
    Returns the path for a handler.
    Example usage:
    >>> reverse_url('login')
    /account/login
    """
    for path, handler in __request__handlers__.items():
        if handler.__name__ == handler_name:
            return path
    raise ValueError('No handler named %s' % handler_name)

def application(environ, start_response):
    """
    Searches for a handler for a certain request and
    dispatches it if found. Returns 404 if not found.
    """

    request = create_request_path(environ.copy())
    complete_path = '/%s' % '/'.join(request)
    handler = find_request_handler(complete_path)
    if not handler:
        error = Http404()
        if settings.DEBUG:
            response = error.get_response(environ['PATH_INFO'],
                    details="Debug: No handler for path %s" % complete_path)
        else:
            response = error.get_response(environ['PATH_INFO'])
        start_response(response.status_code, response.headers)
        return [response.content]

    try:
        req = Request(environ)
        try:
            response = handler(req)
        except HttpException, exception:
            response = exception.get_response(environ['PATH_INFO'])
        headers = response.headers + response.cookies.cookie_headers
        start_response(response.status_code, headers)
        if isinstance(response.content, basestring):
            return [response.content]
        else:
            return response.content
    except Exception, exception:
        error = Http500()
        if settings.DEBUG:
            exception_type, exception_value, exception_trace = sys.exc_info()
            trace = "".join(traceback.format_exception(exception_type,
                                                    exception_value,
                                                    exception_trace))
            response = error.get_response(
                    environ['PATH_INFO'],
                    description="%s: %s" % (exception.__class__.__name__,
                                            exception),
                    traceback=trace)
        else:
            response = error.get_response(environ['PATH_INFO'])
        start_response(response.status_code, response.headers)
        return [response.content]
